#
# Copyright (c) Magenta ApS
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

from unittest.mock import patch, MagicMock

import pytest
from parameterized import parameterized
from requests.exceptions import HTTPError
from requests import Response

from os2mo_data_import.utilities import ImportUtility


@parameterized.expand([(0,), (9,)])
def test_retrying_mo_data_insertion_should_pass_in_less_max_10_tries(retries):
    # Arrange
    import_utility = ImportUtility("http://mox.base", "http://mora.base", False)
    import_utility.mo_request_retry_delay = 0.05

    r = Response()
    r.status_code = 200
    mock_json = MagicMock()
    mock_json.return_value = "uuid"
    r.json = mock_json

    mora_helper = MagicMock()
    mora_helper._mo_post.side_effect = retries*[HTTPError()] + [r]
    import_utility.mh = mora_helper

    # Act
    resp = import_utility.insert_mora_data("/some/resource", {})

    # Assert
    assert resp == "uuid"


def test_retrying_mo_data_insertion_should_fail_after_10_retries():
    # Arrange
    import_utility = ImportUtility("http://mox.base", "http://mora.base", False)
    import_utility.mo_request_retry_delay = 0.05

    mora_helper = MagicMock()
    mora_helper._mo_post.side_effect = 10*[HTTPError()]
    import_utility.mh = mora_helper

    # Act + Assert
    with pytest.raises(HTTPError):
        import_utility.insert_mora_data("/some/resource", {})



from unittest import TestCase
from unittest.mock import patch
from uuid import uuid4

from parameterized import parameterized
from requests.exceptions import ConnectionError
from requests.exceptions import HTTPError

from ..mora_helpers import MoraHelper


class MoPostMock:
    def json(self):
        return str(uuid4())

    def raise_for_status(self):
        pass


class TestMoraHelper(TestCase):
    def setUp(self):
        super().setUp()
        self._instance = MoraHelper()

    @parameterized.expand(
        [
            (ConnectionError, 5),
            (HTTPError, 5),
            (Exception, 1),
        ]
    )
    def test_mo_lookup_retries_on_error(self, exception_class, expected_attempts):
        """Check that the expected number of retry attempts are performed, depending
        on the specific exception raised by `requests.get`.
        """
        with self.assertRaises(exception_class):
            with patch("requests.get", side_effect=exception_class):
                self._instance._mo_lookup("not-a-uuid", "not-a-url")
                self.assertEqual(
                    self._instance._mo_lookup.retry.statistics["attempt_number"],
                    expected_attempts,
                )

    # Inputs are titles and user_keys of current class and input. Lastly a bool indicating if it needs to post to MO or not.
    @parameterized.expand(
        [
            # Same title and bvn
            ("Current title", "Current bvn", "Current title", "Current bvn", False),
            # Same title, other bvn
            ("Current title", "Current bvn", "Current title", "Another bvn", False),
            # Same bvn, other title
            ("Current title", "Current bvn", "Another title", "Current bvn", False),
            # Other title and bvn
            ("Current title", "Current bvn", "Another title", "Another bvn", True),
        ]
    )
    def test_ensure_class(self, name, user_key, title, bvn, expected_post):
        """Test to check behavior of ensure_class_in_facet"""
        current_class_uuid = uuid4()
        org = uuid4
        owner = uuid4()
        with patch.object(
            self._instance,
            "read_classes_in_facet",
            return_value=[
                [{"uuid": str(current_class_uuid), "name": name, "user_key": user_key}],
                "",
            ],
        ):
            with patch.object(
                self._instance, "read_organisation", return_value=str(org)
            ):
                with patch.object(
                    self._instance, "_mo_post", return_value=MoPostMock()
                ) as _mo_post_mock:
                    return_uuid = self._instance.ensure_class_in_facet(
                        "facet_name", bvn, title=title, owner=owner
                    )
        if expected_post:
            _mo_post_mock.assert_called_once_with(
                "f/facet_name/",
                {
                    "name": title,
                    "user_key": bvn,
                    "scope": "TEXT",
                    "org_uuid": str(org),
                    "owner": str(owner),
                },
            )
            assert return_uuid != current_class_uuid
        else:
            _mo_post_mock.assert_not_called()
            assert return_uuid == current_class_uuid



import asyncio
import collections
from functools import wraps


def async_to_sync(f):
    """Decorator to run an async function to completion.

    Example:

        @async_to_sync
        async def sleepy(seconds):
            await sleep(seconds)

        sleepy(5)
    
    Args:
        f (async function): The async function to wrap and make synchronous.

    Returns:
        :obj:`sync function`: The syncronhous function wrapping the async one.
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        loop = asyncio.get_event_loop()
        future = asyncio.ensure_future(f(*args, **kwargs))
        return loop.run_until_complete(future)

    return wrapper


def dict_map(func, dicty):
    def unfold_call(entry):
        return func(entry[0], entry[1])

    return dict(map(unfold_call, dicty.items()))



#
# Copyright (c) Magenta ApS
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

import logging
from uuid import uuid4, UUID
from requests import Session, HTTPError
from datetime import datetime, timedelta

from os2mo_data_import.mora_data_types import (
    OrganisationUnitType,
    TerminationType,
    EngagementTerminationType,
    EmployeeType
)

from os2mo_data_import.mox_data_types import (
    Organisation,
    Klassifikation,
    Facet,
    Klasse,
    Itsystem
)
from os2mo_helpers.mora_helpers import MoraHelper
from tenacity import stop_after_attempt, wait_fixed, RetryError, Retrying

logger = logging.getLogger("moImporterUtilities")


class ImportUtility(object):
    """
    ImportUtility
    TODO: This class relies heavily on asserts, which are gennerally not wanted in
    this context. We should create meaningfull exceptions and raise these when
    needed.
    """

    def __init__(self, mox_base, mora_base, demand_consistent_uuids, dry_run=False):

        # Import Params
        self.demand_consistent_uuids = demand_consistent_uuids

        # Service endpoint base
        self.mox_base = mox_base
        self.mora_base = mora_base

        # Session
        self.mh = MoraHelper(self.mora_base, use_cache=False)
        self.session = Session()

        # Placeholder for UUID import
        self.organisation_uuid = None

        # Existing UUIDS
        # TODO: More elegant version of this please
        self.existing_uuids = []

        # UUID map
        self.inserted_organisation = {}
        self.inserted_facet_map = {}
        self.inserted_klasse_map = {}
        self.inserted_itsystem_map = {}
        self.inserted_org_unit_map = {}
        self.inserted_employee_map = {}

        # Hard-coded for now
        self.mo_request_retries = 10
        self.mo_request_retry_delay = 3  # seconds

        # Deprecated
        self.dry_run = dry_run

    def import_organisation(self, reference, organisation):
        """
        Convert organisation to OIO formatted post data
        and import into the MOX datastore.

        :param str reference: Reference to the user defined identifier
        :param object organisation: Organisation object

        :returns: Inserted UUID
        :rtype: str/uuid
        """

        if not isinstance(organisation, Organisation):
            raise TypeError("Not of type Organisation")

        resource = "organisation/organisation"

        payload = organisation.build()

        if organisation.uuid is not None:
            organisation_uuid = organisation.uuid

        self.organisation_uuid = self.insert_mox_data(
            resource=resource,
            data=payload,
            uuid=organisation_uuid
        )

        # Global validity
        self.date_from = organisation.date_from
        self.date_to = organisation.date_to

        return self.organisation_uuid

    def import_klassifikation(self, reference, klassifikation):
        """
        Begin import of klassifikation

        :param str reference: Reference to the user defined identifier
        :param object klassifikation: Klassifikation object

        :returns: Inserted UUID
        :rtype: str/uuid
        """
        if not isinstance(klassifikation, Klassifikation):
            raise TypeError("Not of type Klassifikation")

        resource = "klassifikation/klassifikation"

        klassifikation.organisation_uuid = self.organisation_uuid

        payload = klassifikation.build()

        self.klassifikation_uuid = self.insert_mox_data(
            resource=resource,
            data=payload,
        )

        return self.klassifikation_uuid

    def import_facet(self, reference, facet):
        """
        Begin import of facet

        :param str reference: Reference to the user defined identifier
        :param object facet: Facet object

        :returns: Inserted UUID
        :rtype: str/uuid
        """

        if not isinstance(facet, Facet):
            raise TypeError("Not of type Facet")

        resource = "klassifikation/facet"

        facet.organisation_uuid = self.organisation_uuid
        facet.klassifikation_uuid = self.klassifikation_uuid

        # NEED TO BE FIXED
        facet.date_from = self.date_from
        facet.date_to = self.date_to

        payload = facet.build()

        self.inserted_facet_map[reference] = self.insert_mox_data(
            resource=resource,
            data=payload,
        )

        return self.inserted_facet_map[reference]

    def import_klasse(self, reference, klasse):
        """
        Insert a klasse object

        Begin import of klassifikation

        :param str reference: Reference to the user defined identifier
        :param object organisation: Organisation object

        :returns: Inserted UUID
        :rtype: str/uuid
        """

        if not isinstance(klasse, Klasse):
            raise TypeError("Not of type Facet")

        uuid = klasse.uuid
        facet_ref = klasse.facet_type_ref

        facet_uuid = self.inserted_facet_map.get(facet_ref)

        if not facet_uuid:
            error_message = "Facet ref: {} does not exist for {}".format(
                facet_ref, klasse
            )
            logger.error(error_message)
            raise KeyError(error_message)

        klasse.organisation_uuid = self.organisation_uuid
        klasse.facet_uuid = facet_uuid
        klasse.date_from = self.date_from
        klasse.date_to = self.date_to

        if uuid is None:
            klasse_uuid = None
        else:
            klasse_uuid = uuid

        payload = klasse.build()

        import_uuid = self.insert_mox_data(
            resource="klassifikation/klasse",
            data=payload,
            uuid=klasse_uuid
        )
        assert(uuid is None or import_uuid == str(klasse_uuid))
        self.inserted_klasse_map[reference] = import_uuid

        return self.inserted_klasse_map[reference]

    def import_itsystem(self, reference, itsystem):
        """
        Create IT System

        :param str reference: Reference to the user defined identifier
        :param object organisation: Organisation object

        :returns: Inserted UUID
        :rtype: str/uuid
        """

        if not isinstance(itsystem, Itsystem):
            raise TypeError("Not of type Itsystem")

        resource = 'organisation/itsystem'

        itsystem.organisation_uuid = self.organisation_uuid
        itsystem.date_from = self.date_from
        itsystem.date_to = self.date_to

        payload = itsystem.build()

        self.inserted_itsystem_map[reference] = self.insert_mox_data(
            resource=resource,
            data=payload,
        )

        return self.inserted_itsystem_map[reference]

    def import_org_unit(self, reference, organisation_unit, details=[]):
        """
        Insert organisation unit and details

        .. note::
            Optional data objects are relational objects which
            belong to the organisation unit, such as an address type

        :param str reference: Reference to the user defined identifier
        :param object organisation_unit: Organisation object
        :param list details: List of details
        :returns: Inserted UUID
        :rtype: str/uuid
        """

        if not isinstance(organisation_unit, OrganisationUnitType):
            raise TypeError("Not of type OrganisationUnitType")

        if reference in self.inserted_org_unit_map:
            return False

        resource = 'organisation/organisationenhed'

        # payload = self.build_mo_payload(organisation_unit_data)
        parent_ref = organisation_unit.parent_ref
        if parent_ref:
            parent_uuid = self.inserted_org_unit_map.get(parent_ref)
            organisation_unit.parent_uuid = parent_uuid

        if not organisation_unit.parent_uuid:
            organisation_unit.parent_uuid = self.organisation_uuid

        type_ref_uuid = self.inserted_klasse_map.get(
            organisation_unit.type_ref
        )

        if hasattr(organisation_unit, "time_planning_ref"):
            organisation_unit.time_planning_ref_uuid = self.inserted_klasse_map.get(
                organisation_unit.time_planning_ref
            )

        if hasattr(organisation_unit, 'org_unit_level_ref'):
            organisation_unit.org_unit_level_uuid = self.inserted_klasse_map.get(
                organisation_unit.org_unit_level_ref
            )

        organisation_unit.type_ref_uuid = type_ref_uuid

        payload = organisation_unit.build()

        if 'uuid' in payload:
            if payload['uuid'] in self.existing_uuids:
                logger.info('Re-import org-unit: {}'.format(payload['uuid']))
                re_import = 'NO'
                resource = 'details/edit'
                payload_keys = list(payload.keys())
                payload['data'] = {}
                for key in payload_keys:
                    payload['data'][key] = payload[key]
                    del payload[key]
                payload['type'] = 'org_unit'
            else:
                re_import = 'NEW'
                logger.info('New unit - Forced uuid: {}'.format(payload['uuid']))
                resource = 'ou/create'
        else:
            re_import = 'NEW'
            logger.info('New unit, random uuid')
            resource = 'ou/create'

        logger.debug('Unit payload: {}'.format(payload))

        uuid = self.insert_mora_data(
            resource=resource,
            data=payload
        )

        if 'uuid' in payload:
            assert (uuid == payload['uuid'])
        if not uuid:
            raise ConnectionError("Something went wrong")

        # Add to the inserted map
        self.inserted_org_unit_map[reference] = uuid

        data = {}
        data['address'] = self.mh._get_detail(uuid, 'address', object_type='ou')

        # Build details (if any)
        details_payload = []

        for detail in details:
            detail.org_unit_uuid = uuid

            date_from = detail.date_from

            if not date_from:
                date_from = organisation_unit.date_from

            build_detail = self.build_detail(
                detail=detail
            )

            if not build_detail:
                continue

            found_hit = self._payload_compare(build_detail, data)
            logger.debug('Found hit for {}: {}'.format(uuid, found_hit))
            if not found_hit and re_import == 'NO':
                re_import = 'YES'

            # TODO: SHOLD WE UPDATE FROM TO TODAY IN CASE OF RE-IMPORT?
            if re_import == 'YES':
                valid_from = datetime.now().strftime('%Y-%m-%d')  # today
                build_detail['validity']['from'] = valid_from
            details_payload.append(build_detail)

        if re_import == 'YES':
            logger.info('Terminating details for unit {}'.format(uuid))
            for item in data['address']:
                self._terminate_details(item['uuid'], 'address')
        if re_import in ('YES', 'NEW'):
            logger.info('Re-import unit: {}'.format(re_import))
            self.insert_mora_data(
                resource="details/create",
                data=details_payload
            )

        return uuid

    def import_employee(self, reference, employee, details=[]):
        """
        Import employee

        :param str reference: Reference to the user defined identifier
        :param object employee: Employee object
        :param list details: List of details
        :returns: Inserted UUID
        :rtype: str/uuid
        """
        if not isinstance(employee, EmployeeType):
            raise TypeError("Not of type EmployeeType")

        employee.org_uuid = self.organisation_uuid
        payload = employee.build()

        if 'uuid' in payload and payload['uuid'] in self.existing_uuids:
            logger.info('Re-import employee {}'.format(payload['uuid']))
            re_import = 'NO'
        else:
            re_import = 'NEW'
            logger.info('New employee, uuid {}'.format(payload.get('uuid')))

        # We unconditionally create or update the user, this should
        # ensure that we are always updated with correct current information.
        mora_resource = "e/create"
        uuid = self.insert_mora_data(
            resource=mora_resource,
            data=payload,
        )

        # Add uuid to the inserted employee map
        self.inserted_employee_map[reference] = uuid

        data = {}
        data['it'] = self.mh._get_detail(uuid, 'it')
        data['role'] = self.mh._get_detail(uuid, 'role')
        data['leave'] = self.mh._get_detail(uuid, 'leave')
        data['address'] = self.mh._get_detail(uuid, 'address')
        data['manager'] = self.mh._get_detail(uuid, 'manager')
        data['engagement'] = self.mh._get_detail(uuid, 'engagement')
        data['association'] = self.mh._get_detail(uuid, 'association')

        # In case of en explicit termination, we terminate the employee or
        # employment and return imidiately.
        for detail in details:
            if isinstance(detail, TerminationType):
                logger.info(
                    'Explicit termination of eng {} from {}'.format(
                        uuid, detail.date_from
                    )
                )
                self._terminate_employee(uuid, date_from=detail.date_from)
                return uuid
            if isinstance(detail, EngagementTerminationType):
                logger.info(
                    'Explicit termination of eng {}'.format(
                        detail.engagement_uuid
                    )
                )
                self._terminate_details(detail.engagement_uuid, 'engagement')
                return uuid

        if details:
            additional_payload = []
            for detail in details:

                if not detail.date_from:
                    detail.date_from = self.date_from

                # Create payload (as dict)
                detail_payload = self.build_detail(
                    detail=detail,
                    employee_uuid=uuid
                )

                if not detail_payload:
                    continue

                # If we do not have existing data, the new data should be imported
                if len(data[detail_payload['type']]) == 0 and re_import == 'NO':
                    re_import = 'UPDATE'
                elif data[detail_payload['type']]:
                    found_hit = self._payload_compare(detail_payload, data)
                    if not found_hit:
                        re_import = 'YES'
                additional_payload.append(detail_payload)

            for item in additional_payload:
                valid_from = item['validity']['from']
                valid_to = item['validity']['to']
                now = datetime.now()
                py_from = datetime.strptime(valid_from, '%Y-%m-%d')
                if valid_to is not None:
                    py_to = datetime.strptime(valid_to, '%Y-%m-%d')
                else:
                    py_to = datetime.strptime('2200-01-01', '%Y-%m-%d')

                logger.debug(
                    'Py-from:{}, Py-to:{}, Now:{}'.format(py_from, py_to, now)
                )
                if re_import == 'YES' and py_from < now and py_to > now:
                    logger.debug('Updating valid_from')
                    valid_from = datetime.now().strftime('%Y-%m-%d')  # today
                    item['validity']['from'] = valid_from

            logger.info('Re-import: {}'.format(re_import))

            if re_import == 'YES':
                logger.info('Non-explicit termination: {}'.format(uuid))
                self._terminate_employee(uuid)

            if re_import in ('YES', 'NEW', 'UPDATE'):
                self.insert_mora_data(
                    resource="details/create",
                    data=additional_payload
                )

        return uuid

    def build_detail(self, detail, employee_uuid=None):
        """
        Build detail payload

        :param MoType detail: Detail object

        .. note::
            A detail can be one of the following types:

                - address
                - asso
                - role
                - itsystem
                - engagement
                - manager

            :Reference: :mod:`os2mo_data_import.mora_data_types`

        :param str employee_uuid: (Option) Employee uuid if it exists

        :return: Detail POST data payload
        :rtype: dict
        """

        if employee_uuid:
            detail.person_uuid = employee_uuid

        common_attributes = [
            ("type_ref", "type_ref_uuid"),
            ("job_function_ref", "job_function_uuid"),
            ("address_type_ref", "address_type_uuid"),
            ("manager_level_ref", "manager_level_uuid")
        ]

        for check_value, set_value in common_attributes:
            if not hasattr(detail, check_value):
                continue

            uuid = self.inserted_klasse_map.get(
                getattr(detail, check_value)
            )

            setattr(detail, set_value, uuid)

        # Uncommon attributes
        if hasattr(detail, "visibility_ref"):
            detail.visibility_ref_uuid = self.inserted_klasse_map.get(
                detail.visibility_ref
            )

        if hasattr(detail, 'primary_ref'):
            detail.primary_uuid = self.inserted_klasse_map.get(
                detail.primary_ref
            )

        if hasattr(detail, "org_unit_ref"):
            detail.org_unit_uuid = self.inserted_org_unit_map.get(
                detail.org_unit_ref
            )

        if hasattr(detail, "organisation_uuid"):
            detail.organisation_uuid = self.organisation_uuid

        if hasattr(detail, "itsystem_ref"):
            detail.itsystem_uuid = self.inserted_itsystem_map.get(
                detail.itsystem_ref
            )

        if hasattr(detail, "responsibilities"):
            detail.responsibilities = [
                self.inserted_klasse_map[reference]
                for reference in detail.responsibility_list
            ]

        return detail.build()

    def insert_mox_data(self, resource, data, uuid=None):
        service_url = f"{self.mox_base}/{resource}"

        if uuid:
            update_url = "{service}/{uuid}".format(
                service=service_url,
                uuid=uuid
            )

            response = self.session.put(
                url=update_url,
                json=data
            )

            if response.status_code != 200:
                logger.error(
                    'Mox put. Response: {}, data'.format(response.text, data)
                )
                raise HTTPError("Inserting mox data failed")

        else:
            response = self.session.post(
                url=service_url,
                json=data
            )

            if response.status_code != 201:
                logger.error(
                    'Mox post. Response: {}, data'.format(response.text, data)
                )

                raise HTTPError("Inserting mox data failed")

        response_data = response.json()
        return response_data["uuid"]

    def insert_mora_data(self, resource, data, uuid=None):

        # TESTING
        if self.dry_run:
            uuid = uuid4()
            return str(uuid)

        # Try calling MO 10 times one second apart before giving up
        # This is necessary due to sporadic and temporary failures on the network
        try:
            for attempt in Retrying(
                stop=stop_after_attempt(self.mo_request_retries), wait=wait_fixed(self.mo_request_retry_delay)
            ):
                with attempt:

                    response = self.mh._mo_post(
                        url=resource,
                        payload=data,
                        force=True,
                    )

                    if response.status_code == 400:
                        error = response.json()['description']
                        if error.find('does not give raise to a new registration') > 0:
                            uuid_start = error.find('with id [')
                            uuid = error[uuid_start+9:uuid_start+45]
                            try:
                                UUID(uuid, version=4)
                            except ValueError:
                                raise Exception('Unable to read uuid')
                        else:
                            logger.error(
                                'MO post. Error: {}, data: {}'.format(error, data)
                            )
                            raise HTTPError("Inserting mora data failed")
                    elif response.status_code not in (200, 201):
                        logger.error(
                            'MO post. Response: {}, data: {}'.format(response.text, data)
                        )
                        raise HTTPError("Inserting mora data failed")
                    else:
                        uuid = response.json()
                    return uuid

                logger.debug("Error inserting data to MO... trying again")

        except RetryError as retry_error:
            retry_error.reraise()

    def _terminate_employee(self, uuid, date_from=None):
        endpoint = 'e/{}/terminate'
        yesterday = datetime.now() - timedelta(days=1)
        if date_from:
            to = date_from
        else:
            to = yesterday.strftime('%Y-%m-%d')

        payload = {
            'terminate_all': True,
            'validity': {
                'to': to
            }
        }
        resource = endpoint.format(uuid)

        self.insert_mora_data(
            resource=resource,
            data=payload
        )
        return uuid

    def _terminate_details(self, uuid, detail_type):
        logger.info('Terminate detail {}:  {}'.format(uuid, detail_type))
        yesterday = datetime.now() - timedelta(days=1)
        payload = {
            'type': detail_type,
            'uuid': uuid,
            'validity': {
                'to': yesterday.strftime('%Y-%m-%d')
            }
        }
        logger.debug('Terminate detail payload: {}'.format(payload))
        try:
            self.insert_mora_data(
                resource='details/terminate',
                data=payload
            )
        except HTTPError:
            logger.info('Tried to terminate a non-existing uuid')
        return uuid

    def _std_compare(self, item_payload, data_item, extra_field=None):
        """ Helper for _payload_compare, performs the checks that are identical
        for most object types.
        :param item_payload: The new payload data.
        :param data_item: The existing set of data.
        :param extra_field: If not None the comparison will also be done on this
        field, otherwise the comparison is only performed on uuid and validity.
        :return: True if identical, otherwise False
        """
        identical = (
            (data_item['org_unit']['uuid'] == item_payload['org_unit']['uuid']) and
            (data_item['validity']['from'] == item_payload['validity']['from']) and
            (data_item['validity']['to'] == item_payload['validity']['to'])
        )
        if extra_field is not None:
            identical = (
                identical and
                data_item[extra_field]['uuid'] == item_payload[extra_field]['uuid']
            )
        return identical

    def _payload_compare(self, item_payload, data):
        """ Compare an exising data-set with a new payload and tell whether
        the new payload is different from the exiting data.
        :param item_payload: New the payload data.
        :param data_item: The existing set of data.
        :param extra_field: If not None the comparison will also be done on this
        field, otherwise the comparison is only performed on uuid and validity.
        :return: True if identical, otherwise False
        """
        data_type = item_payload['type']

        logger.debug(
            'Payload compare. item_payload: {}, data: {}'.format(item_payload, data)
        )

        found_hit = False
        if data_type == 'engagement':
            # In priciple, we should be able to re-calculate the hash stored in
            # integration data and compare directly from that.
            for data_item in data[data_type]:
                if (self._std_compare(item_payload, data_item, 'job_function') and
                    item_payload['user_key'] == data_item['user_key'] and
                    item_payload['fraction'] == data_item['fraction']):
                    found_hit = True

        elif data_type == 'role':
            for data_item in data[data_type]:
                if self._std_compare(item_payload, data_item, 'role_type'):
                    found_hit = True
                    logger.debug('Found hit in role')
        elif data_type == 'leave':
            for data_item in data[data_type]:
                if ((data_item['validity']['from'] ==
                     item_payload['validity']['from']) and

                    (data_item['validity']['to'] ==
                     item_payload['validity']['to'])):
                    found_hit = True
                    logger.debug('Found hit in leave')
        elif data_type == 'it':
            for data_item in data[data_type]:
                if (
                    (data_item['validity']['from'] ==
                     item_payload['validity']['from']) and

                    (data_item['validity']['to'] ==
                     item_payload['validity']['to']) and

                    (data_item['itsystem']['uuid'] ==
                     item_payload['itsystem']['uuid'])
                ):
                    found_hit = True
                    logger.debug('Found hit in it')
        elif data_type == 'address':
            for data_item in data[data_type]:
                if (
                    (data_item['validity']['from'] ==
                     item_payload['validity']['from']) and

                    (data_item['validity']['to'] ==
                     item_payload['validity']['to']) and

                    (data_item['value'] == item_payload['value'])
                ):
                    found_hit = True
                    logger.debug('Found hit in adress')
        elif data_type == 'manager':
            for data_item in data[data_type]:
                identical = self._std_compare(item_payload, data_item,
                                              'manager_level')
                uuids = []
                for item in item_payload['responsibility']:
                    uuids.append(item['uuid'])
                for responsibility in data_item['responsibility']:
                    identical = identical and (responsibility['uuid'] in uuids)
                identical = (identical and
                             (len(data_item['responsibility']) == len(uuids)))
                if identical:
                    found_hit = True
                    logger.debug('Found  hit in manager')
        elif data_type == 'association':
            for data_item in data[data_type]:
                if self._std_compare(item_payload, data_item, 'association_type'):
                    found_hit = True
                    logger.debug('Found hit in association')
        else:
            raise Exception('Uknown detail!')
        logger.info('Found hit: {}'.format(found_hit))
        return found_hit



#
# Copyright (c) Magenta ApS
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import logging

logger = logging.getLogger("moImporterMoxTypes")


class Base(object):
    """
    Base class for all MOX type objects

    .. note::
        An arbitrary json string with relationships between
        user defined identifiers and the actual object uuids (str)

        This functionality facilitates the option to perform
        multiple imports wihtout creating duplicate objects.

    :param str date_from: Start date e.g. "1982-01-01"
    :param str date_to: End date e.g. "infinity"
    """
    def __init__(self):

        self.date_from = "1930-01-01"
        self.date_to = "infinity"

    def create_validity(self):
        """
        Create validity key/value pair from date_from and date_to.

        :return: Validity
        :rtype: dict
        """

        if not self.date_from or not self.date_to:
            raise AssertionError("Date is not specified, cannot create validity")

        return {
            "from": self.date_from,
            "to": self.date_to
        }

    def __repr__(self):
        return str(self.__dict__)


class Facet(Base):
    """
    Facet type - parent of klasse type objects.

    In the current implementation of the os2mo application
    no other types than default values should be needed.

    :param uuid: The object uuid
    :param str date_from: Start date e.g. "1982-01-01"
    :param str date_to: End date e.g. "infinity"
    """

    def __init__(self, user_key, uuid=None, organisation_uuid=None,
                 klassifikation_uuid=None, date_from=None, date_to=None):

        # Init parent
        super().__init__()

        self.uuid = uuid
        self.user_key = user_key

        self.organisation_uuid = str(organisation_uuid)
        self.klassifikation_uuid = str(klassifikation_uuid)

        self.date_from = (date_from or "1930-01-01")
        self.date_to = (date_to or "infinity")

    def build(self):
        """
        Build a MOX POST data payload

        :return: POST data payload
        :rtype: dict
        """

        properties = {
            "brugervendtnoegle": self.user_key,
            "virkning": self.create_validity()
        }

        attributter = {
            "facetegenskaber": [properties]
        }

        relationer = {
            "ansvarlig": [
                {
                    "objekttype": "organisation",
                    "uuid": self.organisation_uuid,
                    "virkning": self.create_validity()
                }
            ],
            "facettilhoerer": [
                {
                    "objekttype": "klassifikation",
                    "uuid": self.klassifikation_uuid,
                    "virkning": self.create_validity()
                }
            ]
        }

        tilstande = {
            "facetpubliceret": [
                {
                    "publiceret": "Publiceret",
                    "virkning": self.create_validity()
                }
            ]
        }

        return {
            "attributter": attributter,
            "relationer": relationer,
            "tilstande": tilstande
        }


class Klasse(Base):
    """
    The Klasse type provides functionality for collecting
    user defined klasse meta objects for import.

    In the current implementation of the os2mo application
    2 Klasse objects must be created:

    .. code-block:: text

        "user_key: "Telefon",
        "facet_type_ref": "org_unit_address_type",
        ...

        "user_key: "AdressePost",
        "facet_type_ref": "org_unit_address_type",
        ...

    As hinted in the example above, these objects belong to
    (facet parent) org_unit_address_type.

    :param str/uuid uuid: The object uuid
    :param str title: Displayed title value
    :param str user_key: (Optional) user key for internal reference
    :param str scope: Scope type value

    .. important::
        The scope determines which field type should be displayed
        in the frontend application.

        Must be one of the following values:

            - DAR,
            - EAN,
            - EMAIL,
            - PHONE,
            - PNUMBER,
            - TEXT,
            - WWW

    :param str example: An example showcasing the format of the value.

    .. note::
        This is currently not displayed in the frontend application.

        .. code-block:: text

            {
                ...
                "example": "<UUID>"
            }

    :param str description: A description paragraph

    .. note::
        This is currently not displayed in the frontend application.

        .. code-block:: text

            {
                ...
                "description": "Please write something beautiful"
            }

    :param str facet_type_ref: Reference to the parent Facet type
    :param str date_from: Start date e.g. "1982-01-01"
    :param str date_to: End date e.g. "infinity"
    """

    def __init__(self, facet_type_ref, user_key, description=None,
                 example=None, scope=None, title=None, uuid=None,
                 date_from=None, date_to=None):
        super().__init__()

        self.facet_type_ref = facet_type_ref
        self.user_key = user_key
        self.description = description
        self.scope = scope
        self.title = (title or user_key)
        self.example = example
        self.uuid = uuid

        self.organisation_uuid = None
        self.facet_uuid = None

        self.date_from = (date_from or "1930-01-01")
        self.date_to = (date_to or "infinity")

    def build(self):
        """
        Build a MOX POST data payload

        :return: POST data payload
        :rtype: dict
        """

        if not self.organisation_uuid:
            raise AssertionError(
                "Organisation uuid is missing - cannot build"
            )

        if not self.facet_uuid:
            raise AssertionError(
                "Facet uuid is missing - cannot build"
            )

        properties = {
            "brugervendtnoegle": self.user_key,
            "titel": self.title,
            "virkning": self.create_validity()
        }

        # Add all user specified properties
        if self.description:
            properties["beskrivelse"] = self.description

        if self.scope:
            properties["omfang"] = self.scope

        if self.example:
            properties["eksempel"] = self.example

        attributter = {
            "klasseegenskaber": [properties]
        }

        relationer = {
            "ansvarlig": [
                {
                    "objekttype": "organisation",
                    "uuid": self.organisation_uuid,
                    "virkning": self.create_validity()
                }
            ],
            "facet": [
                {
                    "objekttype": "facet",
                    "uuid": self.facet_uuid,
                    "virkning": self.create_validity()
                }
            ]
        }

        tilstande = {
            "klassepubliceret": [
                {
                    "publiceret": "Publiceret",
                    "virkning": self.create_validity()
                }
            ]
        }

        payload = {
            "attributter": attributter,
            "relationer": relationer,
            "tilstande": tilstande
        }
        logger.debug('Klasse payload: {}'.format(payload))
        return payload


class Itsystem(Base):
    """
    Itsystem class for creating it systems which belong to the organisation.
    Employees can be tied to an it system with a user name.

    :param str system_name: Name of the it system

    Examples: "Main computer", "Employee database" etc.

    :param str user_key: Username or user reference
    """

    def __init__(self, system_name, user_key=None):
        super().__init__()

        self.system_name = system_name
        self.user_key = (user_key or system_name)

        self.organisation_uuid = None

    def build(self):
        """
        Build a MOX POST data payload

        :return: POST data payload
        :rtype: dict
        """
        properties = {
            "brugervendtnoegle": self.user_key,
            "itsystemnavn": self.system_name,
            "virkning": self.create_validity()
        }

        attributter = {
            "itsystemegenskaber": [properties]
        }

        relationer = {
            "tilhoerer": [
                {
                    "uuid": self.organisation_uuid,
                    "virkning": self.create_validity()
                }
            ]
        }

        tilstande = {
            "itsystemgyldighed": [
                {
                    "gyldighed": "Aktiv",
                    "virkning": self.create_validity()
                }
            ]
        }

        return {
            "attributter": attributter,
            "relationer": relationer,
            "tilstande": tilstande
        }


class Klassifikation(Base):
    """
    The Klassifikation object is the parent of all Facet types
    within an organisation.

    The Klassifikation object is automatically created
    with both user_key and parent_name set to the name of the organisation.

    :Reference: :class:`os2mo_data_import.helpers.ImportHelper.add_organisation`

    :param str user_key: Name of the parent organisation
    :param str parent_name: Name of the parent organisation or alias
    :param str description: A description paragraph
    :param str date_from: Start date e.g. "1982-01-01"
    :param str date_to: End date e.g. "infinity"
    """

    def __init__(self, user_key, parent_name, description,
                 date_from=None, date_to=None):

        # Init parent
        super().__init__()

        self.user_key = user_key
        self.description = description
        self.parent_name = parent_name

        self.organisation_uuid = None
        self.date_from = (date_from or "1930-01-01")
        self.date_to = (date_to or "infinity")

    def build(self):
        """
        Apply class specific attributes before calling
        the underlying _build method to create the POST data payload.

        :return: POST data payload \
        :rtype: dict
        """
        properties = {
            "brugervendtnoegle": self.user_key,
            "beskrivelse": self.description,
            "kaldenavn": self.parent_name,
            "virkning": self.create_validity()
        }

        attributter = {
            "klassifikationegenskaber": [properties]
        }

        relationer = {
            "ansvarlig": [
                {
                    "objekttype": "organisation",
                    "uuid": self.organisation_uuid,
                    "virkning": self.create_validity()
                }
            ],
            "ejer": [
                {
                    "objekttype": "organisation",
                    "uuid": self.organisation_uuid,
                    "virkning": self.create_validity()
                }
            ]
        }

        tilstande = {
            "klassifikationpubliceret": [
                {
                    "publiceret": "Publiceret",
                    "virkning": self.create_validity()
                }
            ]
        }

        return {
            "attributter": attributter,
            "relationer": relationer,
            "tilstande": tilstande
        }


class Organisation(Base):
    """
    The Organisation object is the top level parent
    for all the underlying object types:

        #. Organisation
        #. Facet
        #. Klasse
        #. Itsystem
        #. Organisation Unit
        #. Employee

    :param str name: Canonical name of the organisation
    :param user_key: (Optional) UUID or logical name for the organisation
    :param str municipality_code: 3-digit municipality code
    :param str/uuid uuid: (Optional) Imported UUID from the source
    :param str date_from: Start date e.g. "1982-01-01"
    :param str date_to: End date e.g. "infinity"
    """

    def __init__(self, name, user_key=None, municipality_code=999,
                 uuid=None, date_from=None, date_to=None):
        super().__init__()

        self.uuid = uuid
        self.name = name
        self.user_key = (user_key or name)
        self.municipality_code = str(municipality_code)

        self.date_from = (date_from or "1930-01-01")
        self.date_to = (date_to or "infinity")

    def build(self):
        """
        Apply class specific attributes before calling
        the underlying _build method to create the POST data payload.

        :return: POST data payload (dict)
        """
        properties = {
            "brugervendtnoegle": self.user_key,
            "organisationsnavn": self.name,
            "virkning": self.create_validity()
        }

        attributter = {
            "organisationegenskaber": [
                properties
            ]
        }

        # Create urn value
        urn_municipality_code = "urn:dk:kommune:{code}".format(
            code=self.municipality_code
        )

        relationer = {
            "myndighed": [
                {
                    "urn": urn_municipality_code,
                    "virkning": self.create_validity()
                }
            ]
        }

        tilstande = {
            "organisationgyldighed": [
                {
                    "gyldighed": "Aktiv",
                    "virkning": self.create_validity()
                }
            ]
        }

        return {
            "attributter": attributter,
            "relationer": relationer,
            "tilstande": tilstande
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kle',
 'mox_helpers',
 'os2mo_data_import',
 'os2mo_data_import.tests',
 'os2mo_helpers',
 'os2mo_helpers.tests']

package_data = \
{'': ['*']}

install_requires = \
['XlsxWriter>=3.0.3,<4.0.0',
 'aiohttp>=3.8.1,<4.0.0',
 'anytree>=2.8.0,<3.0.0',
 'certifi>=2021.10.8,<2022.0.0',
 'chardet>=4.0.0,<5.0.0',
 'click>=8.0.4,<9.0.0',
 'idna>=3.3,<4.0',
 'more-itertools>=8.12.0,<9.0.0',
 'openpyxl>=3.0.9,<4.0.0',
 'requests>=2.27.1,<3.0.0',
 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'os2mo-data-import',
    'version': '3.37.1',
    'description': 'A set of tools for OS2MO data import and export',
    'long_description': '.. _ImportHelper:\n\n**********************\nOS2MO data import tool\n**********************\n\nA small higher level utility for os2mo data import.\n\nThe utility provides functionality to create organisation units\nand employees and to create all of the types to which these can be assigned.\n\nAdditonally the utility can be used to insert data objects\ninto a running instance of os2mo.\n\n\nInstalling\n==========\n\nInstall the os2mo_data_import package as follows:\n\n.. code-block:: bash\n\n    # Checkout the mora source repository\n    https://github.com/OS2mo/os2mo-data-import-and-export\n\n    # Navigate to the local copy of the repository\n    cd /path/to/os2mo-data-import-and-export\n\n    # Install package with pip\n    pip install -e os2mo_data_import\n\n\nGetting started\n===============\nThe main entry point (for most use cases) is the Organisation class,\nwhich functions as a wrapper for all the sub classes.\n\n.. code-block:: python\n\n    # Imports\n    from os2mo_data_import import ImportHelper\n\n    # Init helper\n    os2mo = ImportHelper(create_defaults=True)\n\n    # Add organisation\n    os2mo.add_organisation(\n        identifier="Magenta Aps",\n        user_key="Magenta",\n        municipality_code=101\n    )\n\n\nOrganisation Units\n------------------\nBefore organisation units can be created,\na type for the unit must be added:\n\n.. code-block:: python\n\n    # Add klasse with reference to facet "org_unit_type"\n    os2mo.add_klasse(\n        identifier="Hovedenhed",\n        facet_type_ref="org_unit_type",\n        user_key="D1ED90C5-643A-4C12-8889-6B4174EF4467",\n        title="Hovedenhed"  # This is the displayed value\n    )\n\n\nNow an organisation unit can be added with "org_unit_type_ref" referencing\nthe user defined identifier of the newly created unit type by name:\n\n.. code-block:: python\n\n    # Root unit: Magenta\n    # Belongs to unit type: "Hovedenhed"\n    os2mo.add_organisation_unit(\n        identifier="Magenta",\n        name="Magenta Aps",\n        type_ref="Hovedenhed",  # Reference to the unit type\n        date_from="1986-01-01"\n    )\n\nOrganisation unit "Magenta" is a root unit in this example.\nTo add children/sub units, "Magenta" must be referenced as parent:\n\n.. code-block:: python\n\n    # Add unit type "Afdeling"\n    os2mo.add_klasse(\n        identifier="Afdeling",\n        facet_type_ref="org_unit_type",\n        user_key="91154D1E-E7CA-439B-B910-D4622FD3FD21",\n        title="Afdeling"\n    )\n\n    # Add sub unit "Pilestræde"\n    os2mo.add_organisation_unit(\n        identifier="Pilestræde",\n        type_ref="Afdeling",  # This unit is of type: Afdeling\n        parent_ref="Magenta",  # Sub unit of/Belongs to Magenta\n        date_from="1986-01-01"\n    )\n\n\nOptional data or "details" can be associated with an organisation unit.\n\n.. note::\n\n    At least 2 "Klasse" objects must be created,\n    an object for the primary phone number and\n    an object for the primary mailing address (residence).\n\n    The validation in the (os2mo) frontend application requires:\n\n    The user_key on the the primary phone number object\n    must be specified as "PhoneUnit"\n\n    The user_key on the primary mail address object\n    must be specified as "AddressMailUnit"\n\n    Hence either the "identifier" or the "user_key"\n    must be set to:\n\n        - PhoneUnit\n        - AddressMailUnit\n\n    (The "user_key" is derived from the value of the "identifier\n    if not explicitly set)\n\n    See the example below:\n\n\n.. code-block:: python\n\n    # Add klasse type "AdressePost"\n    # Which belongs to facet type "org_unit_address_type"\n\n    # user_key is not explicitly set, identifier must be "AddressMailUnit"\n    os2mo.add_klasse(\n        identifier="AddressMailUnit",\n        facet_type_ref="org_unit_address_type",\n        title="Adresse",\n        scope="DAR",\n        example="<UUID>"\n    )\n\n    # Add klasse type "Telefon"\n    # Which belongs to facet type "org_unit_address_type"\n\n    # user_key is set to "PhoneUnit", hence the identifier can be anything\n    os2mo.add_klasse(\n        identifier="Telefon",\n        facet_type_ref="org_unit_address_type",\n        user_key="PhoneUnit",\n        title="Tlf",\n        scope="PHONE",\n        example="20304060"\n    )\n\n    # Add "AdressePost" detail to the unit "Magenta"\n    os2mo.add_address_type(\n        organisation_unit="Magenta",\n        value="0a3f50c4-379f-32b8-e044-0003ba298018",\n        type_ref="AdressePost",\n        date_from="1986-01-01"\n    )\n\n    # Add "Telefon" detail to the unit "Magenta"\n    os2mo.add_address_type(\n        organisation_unit="Magenta",\n        value="11223344",\n        type_ref="Telefon",\n        date_from="1986-01-01",\n    )\n\nEmployees\n---------\nEmployees are not directly attached to an organisation unit,\nbut can have a job function which is linked to a unit.\n\nCreate employees first:\n\n.. code-block:: python\n\n    os2mo.add_employee(\n        identifier="Susanne Chæf",\n        cpr_no="0101862233"\n    )\n\n    os2mo.add_employee(\n        identifier="Odin Perskov",\n        cpr_no="0102862234"\n    )\n\nJob function\n------------\nAdd the job function types:\n\n.. code-block:: python\n\n    # Job: CEO ("Direktør")\n    os2mo.add_klasse(\n        identifier="Direktør",\n        facet_type_ref="engagement_type",\n        user_key="Direktør",\n        title="Direktør"\n    )\n\n    # Job: Projectmanager ("Projektleder")\n    os2mo.add_klasse(\n        identifier="Projektleder",\n        facet_type_ref="engagement_type",\n        user_key="Projektleder",\n        title="Projektleder"\n    )\n\n\nAdd job functions to the newly created employees\nwith the "add_type_engagement" method:\n\n.. code-block:: python\n\n    # Susanne Chæf is CEO\n    os2mo.add_engagement(\n        employee="Susanne Chæf",\n        organisation_unit="Magenta",\n        job_function_ref="Direktør",\n        engagement_type_ref="Ansat",\n        date_from="1986-01-01"\n    )\n\n    # Odin Perskov is projectmanager\n    os2mo.add_engagement(\n        employee="Odin Perskov",\n        organisation_unit="Pilestræde",\n        job_function_ref="Projektleder",\n        engagement_type_ref="Ansat",\n        date_from="1986-02-01"\n    )\n\n\nAssociation\n-----------\nIn this example the employee "Odin Perskov" is an external consultant,\nand to reflect this an association type can be assigned:\n\n.. code-block:: python\n\n    os2mo.add_klasse(\n        identifier="Ekstern Konsulent",\n        facet_type_ref="association_type",\n        user_key="F997F306-71DF-477C-AD42-E753F9C21B42",\n        title="Ekstern Konsulent"\n    )\n\n    # Add the consultant association to "Odin Perskov":\n    os2mo.add_association(\n        employee="Odin Perskov",\n        organisation_unit="Pilestræde",\n        job_function_ref="Projektleder",\n        association_type_ref="Ekstern Konsulent",\n        address_uuid="0a3f50c4-379f-32b8-e044-0003ba298018",\n        date_from="1986-10-01"\n    )\n\nIn the following example an address is assigned to employee "Odin Perskov".\nFor residential addresses, valid UUID\'s are used to reference an address\nfrom the "Danish registry of addresses" (DAR):\n\n.. code-block:: python\n\n    # Add address type "AdressePostEmployee"\n    os2mo.add_klasse(\n        identifier="AdressePostEmployee",\n        facet_type_ref="employee_address_type",\n        user_key="2F29C717-5D78-4AA9-BDAE-7CDB3A378018",\n        title="Adresse",\n        scope="DAR",\n        example="<UUID>"\n    )\n\n    # Detail AdressePostEmployee assigned to "Odin Perskov"\n    os2mo.add_address_type(\n        employee="Odin Perskov",\n        value="0a3f50a0-ef5a-32b8-e044-0003ba298018",\n        type_ref="AdressePostEmployee",\n        date_from="1986-11-01",\n    )\n\n\nRoles\n-----\nTo add a role type:\n\n.. code-block:: python\n\n    # A role as contact for external projects\n    os2mo.add_klasse(\n        identifier="Nøgleansvarlig",\n        facet_type_ref="role_type",\n        user_key="0E078F23-A5B4-4FB4-909B-60E49295C5E9",\n        title="Nøgleansvarlig"\n    )\n\n    # Role assigned to "Odin Perskov"\n    os2mo.add_role(\n        employee="Odin Perskov",\n        organisation_unit="Pilestræde",\n        role_type_ref="Nøgleansvarlig",\n        date_from="1986-12-01"\n    )\n\nIt systems\n----------\nGeneric IT systems can be created and assigned to employees with a specified "user_key",\nwhich functions as a reference to a username, pin code etc.:\n\n.. code-block:: python\n\n  # Create IT system: Database\n    os2mo.new_itsystem(\n        identifier="Database",\n        system_name="Database"\n    )\n\n    # Assign access to the database\n    # with username "odpe@db"\n    os2mo.join_itsystem(\n        employee="Odin Perskov",\n        user_key="odpe@db",\n        itsystem_ref="Database",\n        date_from="1987-10-01"\n    )\n\n\nManager type, level and responsibilities\n----------------------------------------\nIn order to assign employees as managers to an organisation unit,\nthe following types must be created:\n\n - manager type\n - manager level\n - A type for each responsibility\n\nCreate manager type:\n\n.. code-block:: python\n\n    os2mo.add_klasse(\n        identifier="Leder",\n        facet_type_ref="manager_type",\n        user_key="55BD7A09-86C3-4E15-AF5D-EAD20EB12F81",\n        title="Virksomhedens direktør"\n    )\n\nCreate manager level:\n\n.. code-block:: python\n\n    os2mo.add_klasse(\n        identifier="Højeste niveau",\n        facet_type_ref="manager_level",\n        user_key="6EAA7DA7-212D-4FD0-A068-BA3F932FDB10",\n        title="Højeste niveau"\n    )\n\nCreate several responsibilities:\n\n.. code-block:: python\n\n    os2mo.add_klasse(\n        identifier="Tage beslutninger",\n        facet_type_ref="responsibility",\n        user_key="A9ABDCCB-EC83-468F-AB7D-175B95E94956",\n        title="Tage beslutninger"\n    )\n\n    os2mo.add_klasse(\n        identifier="Motivere medarbejdere",\n        facet_type_ref="responsibility",\n        user_key="DC475AF8-21C9-4112-94AE-E9FB13FE8D14",\n        title="Motivere medarbejdere"\n    )\n\n    os2mo.add_klasse(\n        identifier="Betale løn",\n        facet_type_ref="responsibility",\n        user_key="0A929060-3392-4C07-8F4E-EF5F9B6AFDE2",\n        title="Betale løn"\n    )\n\nAssign the manager position of Magenta to "Susanne Chæf":\n\n.. code-block:: python\n\n    os2mo.add_manager(\n        employee="Susanne Chæf",\n        organisation_unit="Magenta",\n        manager_type_ref="Leder",\n        manager_level_ref="Højeste niveau",\n        responsibility_list=["Tage beslutninger", "Motivere medarbejdere", "Betale løn"],\n        date_from="1987-12-01",\n    )\n\n\n\nPreservation of UUIDs\n=====================\nIf the system to be imported into MO contains UUIDs that should be preserved in MO,\nit is possible to import the UUIDs for employees, organisational units, classes and\nclassifications. This is achieved by adding an extra uuid argument when creating the\nobject, eg:\n\n.. code-block:: python\n\n    os2mo.add_klasse(\n        identifier="Betale løn",\n        facet_type_ref="responsibility",\n        uuid="195da2b6-e648-4bdc-add1-e22654996997",\n        user_key="0A929060-3392-4C07-8F4E-EF5F9B6AFDE2",\n        title="Betale løn"\n    )\n\n\nExample\n=======\nIf a "real" os2mo application is available,\na practial example is provided with contains similar import data\nas the given examples above.\n\nFeel free to run the "import_example.py" included in the repository:\n\nExample: $os2mo-data-import-and-export/os2mo_data_import/import_example.py\n\nRun example:\n\n.. code-block:: bash\n\n  cd os2mo_data_import\n  python import_example.py\n\n\nReference\n=========\nFor more information on the os2mo project,\nplease refer to the official documentation.\n\nRead the docs: https://os2mo.readthedocs.io\n\n\nKnown Issues\n============\nCurrently it is not possible to assign "Leave" (e.g. various types of leave of absence).\n\nThis issue is related to the validation of type assignments.\n',
    'author': 'Magenta',
    'author_email': 'info@magenta.dk',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://git.magenta.dk/rammearkitektur/os2mo-data-import-and-export',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
